import java.util.ArrayList;
import java.io.*;
import java.util.Calendar;
import java.awt.*;
import javax.swing.*;
import java.util.Map;
import java.util.HashMap;


/**
 *	This class generates answers for Marvin to the user. Methods are usually called from MarvinDialog. All answers
 *	can be generated in either Norwegian or English, given by a boolean param.
 *
 *	@author		Ole Hartvigsen
 */
public class AnswerGenerator{

	private MarvinDialog md;			//Marvin GUI module
	private Context c;
	private Tql tql;
	private A_StarPathPlanner searcher;		//Path Planner

	private JFrame mapFrame;
	private Map locationDescriptions;

	/**
	 *	Simple constructor that sets a referance to a MarvinDialog object and creates a A_StarPathPlanner object
	 *	which is stored in 'searcher'.
	 *
	 *	@param md 		The MarvinDialog object that has created this object.
	 *	@param c		Reference to the context parser
	 *	@param tql		Reference to the TQL parser
	 */
	public AnswerGenerator(MarvinDialog md, Context c, Tql tql){
		this.md = md;
		this.c = c;
		this.tql = tql;
		searcher = new A_StarPathPlanner();
		initMapDescriptions();
	}

	/**
	 *	This method generates answers to person-related questions.
	 *
	 *	@param topicAndType	The question topic and type
	 *	@param record		Person record
	 *
	 *	@return	Answer as String
	 */
	public String generatePersonAnswer(String[] topicAndType, Record record){
		String returnString = "";

		//Specific person informations
		if(topicAndType[1].equals(MarvinDialog.TELEPHONE)){
			returnString += "Du har spurt om telefonnummeret til "+record.getFullName()+".\n";
			if(record.getPhoneNumber()==0) returnString += "Jeg vet ikke hva telefonnummeret til "+record.getFullName()+" er.";
			else returnString += "Telefonnummeret til "+record.getFullName()+" er "+record.getPhoneNumber()+".";
		}
		else if(topicAndType[1].equals(MarvinDialog.ADDRESS)){
			returnString += "Du har spurt om adressen til "+record.getFullName()+".\n";
			if(record.getStreet().equals("")) returnString += "Jeg vet ikke hva adressen til "+record.getFullName()+" er.";
			else returnString += "Adressen til "+record.getFullName()+" er "+record.getStreet()+".";
		}
		else if(topicAndType[1].equals(MarvinDialog.DEPARTMENT)){
			returnString += "Du har spurt om avdelingen til "+record.getFullName()+".\n";
			if(record.getDepartment().equals("")) returnString += "Jeg vet ikke hva avdelingen til "+record.getFullName()+" er.";
			else returnString += "Avdelingen til "+record.getFullName()+" er "+record.getDepartment()+".";
		}
		else if(topicAndType[1].equals(MarvinDialog.TITLE)){
			returnString += "Du har spurt om tittelen til "+record.getFullName()+".\n";
			if(record.getTitle().equals("")) returnString += "Jeg vet ikke hva tittelen til "+record.getFullName()+" er.";
			else returnString += "Tittelen til "+record.getFullName()+" er "+record.getTitle()+".";
		}

		//Location
		else if(topicAndType[1].equals(MarvinDialog.ROUTEPLAN) ||
				topicAndType[1].equals(MarvinDialog.SHOW) ||
				topicAndType[1].equals(MarvinDialog.PLACE) ||
				topicAndType[1].equals(MarvinDialog.GO) ||
				topicAndType[1].equals(MarvinDialog.ACCOMPANY) ||
				topicAndType[1].equals(MarvinDialog.DEFAULT)){
			returnString += "Du har spurt om hvor kontoret til "+record.getFullName()+" er.\n";

			//Determine if we should store the location as a 'memory' answer
			if(!(record.getRoomNumber().equals("") && record.getBuilding().equals(""))){
				md.storeLocation(new Location(record.getRoomNumber(), record.getBuilding()));
			}

			//No room number in record
			if(record.getRoomNumber().equals("")){
				//No building found
				if(record.getBuilding().equals("")){
					returnString += "Jeg vet ikke hvor "+record.getFullName()+" er.";
				}
				//Building found
				else{
					returnString += record.getFullName()+" har kontor p "+record.getBuilding()+".";
					GlosMap tempGM = new GlosMap();
					TrondheimMap tempTM = new TrondheimMap();
					if(tempGM.buildingOnGlos(record.getBuilding())){
						returnString += "\n"+record.getBuilding()+" er her p Glshaugen. P kartet ser du hvor den er.";
						showGlos(record.getBuilding());
					}
					else if(tempTM.buildingInTrondheim(record.getBuilding())){
						returnString += "\nP kartet ser du hvor i Trondheim det er.";
						showTrondheim(record.getBuilding());
					}
					else returnString += "Jeg vet ikke hvor denne bygningen er.";
				}
			}
			//Room number found
			else{
				//IT-bygget
				if(record.getBuilding().equals("IT-bygget")||record.getBuilding().equals("IT-bygg")){
					if(record.getRoomNumber().startsWith("3")){

						//This floor
						if(roomOnThisFloor(record.getRoomNumber())){
							returnString += record.getFullName()+" har kontor p rom "+record.getRoomNumber()+" i denne etasjen.\n";
							returnString += giveDirections(topicAndType[1], record.getRoomNumber());
						}
						else{
							returnString += record.getFullName()+" har kontor p rom "+record.getRoomNumber()+" i denne bygningen.\n";
							returnString += "Jeg vet ikke hvor dette rommet er, men det skal vre i denne etasjen.";
						}
					}
					else{
						String floor;
						if(record.getRoomNumber().startsWith("0")) floor = "kjelleren";
						else floor = record.getRoomNumber().charAt(0)+". etasje";
						returnString += record.getFullName()+" har kontor p rom "+record.getRoomNumber()+" i "+floor+". Du er n i 3. etasje.";
					}
				}
				//No building found
				else if(record.getBuilding().equals("")){
					returnString += record.getFullName()+" har kontor p rom "+record.getRoomNumber()+", men jeg vet ikke i hvilken bygning.";
				}
				//Other building
				else{
					returnString += record.getFullName()+" har kontor p rom "+record.getRoomNumber()+" i "+record.getBuilding()+".";
					//Building on Glshaugen
					GlosMap tempGM = new GlosMap();
					TrondheimMap tempTM = new TrondheimMap();
					if(tempGM.buildingOnGlos(record.getBuilding())){
						returnString += "\n"+record.getBuilding()+" er her p Glshaugen. P kartet ser du hvor den er.";
						showGlos(record.getBuilding());
					}
					else if(tempTM.buildingInTrondheim(record.getBuilding())){
						returnString += "\nP kartet ser du hvor i Trondheim det er.";
						showTrondheim(record.getBuilding());
					}
					else returnString += "Jeg vet ikke hvor denne bygningen er.";
				}
			}
		}
		return returnString;
	}

	/**
	*	Method for generating bus answers
	*/
	public String generateBusAnswer(){
		if(c.getAnswerString().equals("")) return "Telebuster gav ikke noe buss-relatert svar.";
		return c.getAnswerString();
	}

	/**
	*	Method for generating answer based on Telebuster focus
	*/
	public String generateFocusAnswer(){
		if(c.getFocus().equals("sn")) return "Vet du etternavnet?";
		else if(c.getFocus().equals("givenname")) return "Vet du fornavnet?";
		else return "Vennligst gi meg mer informasjon om personen.";
	}

	/**
	*	Method for generating answer based on previously stored location
	*/
	public String generateAnswerFromLocation(String[] topicAndType, Location location){
		String returnString = "";

		if(location.getRoomNumber().equals("")){
			returnString += "Du lurer p hvor "+location.getBuilding()+" er.\n";
			GlosMap tempGM = new GlosMap();
			TrondheimMap tempTM = new TrondheimMap();
			if(tempGM.buildingOnGlos(location.getBuilding())){
				returnString += "Denne bygningen er her p Glshaugen. P kartet ser du hvor den er.";
				showGlos(location.getBuilding());
			}
			else if(tempTM.buildingInTrondheim(location.getBuilding())){
				returnString += "\nP kartet ser du hvor i Trondheim det er.";
				showTrondheim(location.getBuilding());
			}
			else returnString += "Jeg vet ikke hvor denne bygningen er.";
		}
		else{
			returnString += "Du lurer p hvor "+location.getRoomNumber()+" i "+location.getBuilding()+" er.\n";
			GlosMap tempGM = new GlosMap();
			TrondheimMap tempTM = new TrondheimMap();
			if(roomOnThisFloor(location.getRoomNumber()) && (location.getBuilding().equals("")||location.getBuilding().equals("IT-bygget")||location.getBuilding().equals("IT-bygget"))){
				returnString += giveDirections(topicAndType[1], location.getRoomNumber());
			}
			else if(location.getBuilding().equals("IT-bygget")||location.getBuilding().equals("IT-bygg")){
				returnString += "Dette rommet er i denne bygningen, men ikke i denne etasjen.";
			}
			else if(tempGM.buildingOnGlos(location.getBuilding())){
				returnString += "\nDenne bygningen er her p Glshaugen. P kartet ser du hvor den er.";
				showGlos(location.getBuilding());
			}
			else if(tempTM.buildingInTrondheim(location.getBuilding())){
				returnString += "\nP kartet ser du hvor i Trondheim det er.";
				showTrondheim(location.getBuilding());
			}
			else returnString += "Jeg vet ikke hvor denne bygningen er.";
		}

		return returnString;
	}

	/**
	*	Method for generating specific answer
	*/
	public String generateSpecificAnswer(String[] topicAndType){
		String returnString = "";
		String answeredLocation = "";

		if(topicAndType[0].equals("marvin") || topicAndType[0].equals("robot")){
			returnString += "Roboten skal vre ved "+(String)locationDescriptions.get(new Integer(md.getMarvinLocation()))+".";
			answeredLocation = ""+(300+md.getMarvinLocation());
		}
		else if(topicAndType[0].equals("tuc")){
			returnString += "Jeg er her ved inngangen til 3. etasje p IT-bygget";
			answeredLocation = "325";
		}
		else if(topicAndType[0].equals("I") || topicAndType[0].equals("self")){
			returnString += "Du str ved inngangen til 3.etasje p IT-bygget";
			answeredLocation = "325";
		}
		else if(topicAndType[0].equals("exit")){
			returnString += "Utgangen p IT-bygget er i 1. etasje.";
		}
		else if(topicAndType[0].equals("stair")){
			returnString += "Trappen er rett her ved siden av deg.";
			answeredLocation = "325";
		}
		else if(topicAndType[0].equals("lift")){
			answeredLocation = "390";
			returnString += "Du lurer p hvor heisen er.\n";
			returnString += giveDirections(topicAndType[1], answeredLocation);
		}
		else if(topicAndType[0].equals("toilet")){
			returnString += "Du lurer p hvor det nrmeste toalettet er.\n";
			answeredLocation = "335";
			returnString += giveDirections(topicAndType[1], answeredLocation);
		}
		else if(topicAndType[0].equals("gentlementoilet")){
			returnString += "Du lurer p hvor herretoalettet er.\n";
			answeredLocation = "335";
			returnString += giveDirections(topicAndType[1], answeredLocation);
		}
		else if(topicAndType[0].equals("ladiestoilet")){
			returnString += "Du lurer p hvor dametoalettet er.\n";
			answeredLocation = "332";
			returnString += giveDirections(topicAndType[1], answeredLocation);
		}
		else if(topicAndType[0].equals("noon")){
			returnString += "Jeg vet ingenting om middag.\n";
		}
		else if(topicAndType[0].equals("entry")){
			returnString += "Du er ved inngangen til 3. etasje i IT-bygget.";
			returnString += showEntry();
			answeredLocation = "325";
		}
		else if(topicAndType[0].equals("date") || topicAndType[0].equals("day")){
			returnString += answerToday();
		}
		else if(topicAndType[0].equals("clock") || topicAndType[0].equals("time")){
			returnString += answerTime();
		}
		else if(topicAndType[0].equals("roomnumber")){
			String roomNumber = tql.getInverseIs("room");
			if(roomOnThisFloor(roomNumber)){
				answeredLocation = roomNumber;
				returnString += "Du lurer p hvor rom "+answeredLocation+" er.\n";
				returnString += giveDirections(topicAndType[1], answeredLocation);
			}
			else{
				returnString += "Dette rommet finnes ikke i denne etasjen.";
			}
		}
		else if(topicAndType[0].equals("quit")){
			if(mapFrame != null) mapFrame.dispose();
			if(topicAndType[1].equals(MarvinDialog.YOUAREWELCOME)){
				returnString += "Bare hyggelig.";
			}
			else if(topicAndType[1].equals(MarvinDialog.BYE)){
				returnString += "Ha det bra.";
			}
			else{
				returnString += "Ha det fint.";
			}
		}

		//Store location answered
		if(!(answeredLocation.equals(""))){
			md.storeLocation(new Location(answeredLocation, ""));
		}
		return returnString;
	}

	/**
	*	Returns a String about the current time
	*/
	private String answerTime(){
		String returnString = "";

		returnString += "Klokken er ";
		Calendar time = Calendar.getInstance();
		if(time.get(Calendar.HOUR_OF_DAY) >= 10){
			returnString += time.get(Calendar.HOUR_OF_DAY) + ":";
		}
		else{
			returnString += "0" + time.get(Calendar.HOUR_OF_DAY) + ":";
		}
		if(time.get(Calendar.MINUTE) >= 10){
			returnString += time.get(Calendar.MINUTE) + ".";
		}
		else{
			returnString += "0" + time.get(Calendar.MINUTE) + ".";
		}

		return returnString;
	}

	/**
	*	Returns a String about the current date
	*/
	private String answerToday(){
		String returnString = "";

		Calendar time = Calendar.getInstance();
		if(time.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY) returnString += "Det er mandag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY) returnString += "Det er tirsdag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY) returnString += "Det er onsdag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY) returnString += "Det er torsdag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) returnString += "Det er fredag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) returnString += "Det er lrdag ";
		else if(time.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) returnString += "Det er sndag ";

		returnString += "den "+time.get(Calendar.DAY_OF_MONTH)+". ";

		if(time.get(Calendar.MONTH) == Calendar.JANUARY) returnString += "januar.";
		else if(time.get(Calendar.MONTH) == Calendar.FEBRUARY) returnString += "februar.";
		else if(time.get(Calendar.MONTH) == Calendar.MARCH) returnString += "mars.";
		else if(time.get(Calendar.MONTH) == Calendar.APRIL) returnString += "april.";
		else if(time.get(Calendar.MONTH) == Calendar.MAY) returnString += "mai.";
		else if(time.get(Calendar.MONTH) == Calendar.JUNE) returnString += "juni.";
		else if(time.get(Calendar.MONTH) == Calendar.JULY) returnString += "juli.";
		else if(time.get(Calendar.MONTH) == Calendar.AUGUST) returnString += "august.";
		else if(time.get(Calendar.MONTH) == Calendar.SEPTEMBER) returnString += "september.";
		else if(time.get(Calendar.MONTH) == Calendar.OCTOBER) returnString += "oktober.";
		else if(time.get(Calendar.MONTH) == Calendar.NOVEMBER) returnString += "november.";
		else if(time.get(Calendar.MONTH) == Calendar.DECEMBER) returnString += "desember.";

		return returnString;
	}

	/**
	*	Returns a String with directions. Also calls robot guiding if requested from type.
	*/
	private String giveDirections(String type, String destination){
		String returnString = "";

		if(destination.equals("325")){
			returnString += showEntry();
		}
		else{
			int locationAskedFor = Integer.parseInt(destination) - 300;
			if(type.equals(MarvinDialog.ROUTEPLAN) || type.equals(MarvinDialog.SHOW) || type.equals(MarvinDialog.PLACE) || type.equals(MarvinDialog.DEFAULT) || tql.questionClass()==MarvinDialog.EXPLAIN){

				int[] tempCoords = searcher.getRoomCoords(destination);
				if(tempCoords[1] > 180){
					returnString += locationDescriptions.get(new Integer(locationAskedFor))+" ligger nedi gangen, p venstre side";
				}
				else{
					returnString += locationDescriptions.get(new Integer(locationAskedFor))+" ligger nedi gangen, p hyre side";
				}
				ArrayList doors = searcher.getDoorPassages(searcher.findPath(25, locationAskedFor), 25, locationAskedFor);
				while(!doors.isEmpty()){
					int tempDoor = ((Integer)doors.remove(0)).intValue();
					String tempStr = ""+locationDescriptions.get(new Integer(tempDoor));
					tempStr = tempStr.toLowerCase();
					returnString += ", gjennom dren merket "+tempStr;
				}
				returnString += ".\n";
				String tempStr = ""+locationDescriptions.get(new Integer(locationAskedFor));
				tempStr = tempStr.toLowerCase();
				returnString += "P kartet ser du hvor "+tempStr+" er.";
				showIt3(destination);
			}
			else if(type.equals(MarvinDialog.GO) || type.equals(MarvinDialog.ACCOMPANY)){
				String tempStr = ""+locationDescriptions.get(new Integer(locationAskedFor));
				tempStr = tempStr.toLowerCase();
				returnString += "Roboten vil n f beskjed om  g til "+tempStr+", for s  returnere til inngangen.\n";
				returnString += giveRobotTask(destination);
			}
		}

		return returnString;
	}

	/**
	*	Call It3Map to show entry
	*/
	private String showEntry(){
		String returnString = "P kartet ser du hvor du er, ved inngangen til 3. etasje i IT-bygget.";
		showIt3(null);
		return returnString;
	}

	/**
	*	Method for creating a task for the robot
	*/
	public String giveRobotTask(String s){

		//If the robot is not at the entrance
		if(md.getMarvinLocation() != 25){
			return "Roboten er allerede ute p et oppdrag. Den er for tiden ved "+locationDescriptions.get(new Integer(md.getMarvinLocation()))+".";
		}
		else{
			int roomAskedFor = Integer.parseInt(s);

			ArrayList toGoal = searcher.findPath(25, roomAskedFor-300);
			ArrayList fromGoal = searcher.findPath(roomAskedFor-300, 25);
			ArrayList toGoalCommands = searcher.getCommands(toGoal, 25, roomAskedFor-300);
			ArrayList fromGoalCommands = searcher.getCommands(fromGoal, roomAskedFor-300, 25);

			ArrayList c = new ArrayList();
			while(!toGoalCommands.isEmpty()){
				c.add(toGoalCommands.remove(0));
			}
			while(!fromGoalCommands.isEmpty()){
				c.add(fromGoalCommands.remove(0));
			}
			//ArrayList c = searcher.getCommands(toGoal, md.getMarvinLocation(), roomAskedFor-300);

			//Create path-string to write to file, then write it to file named 'path'
			String commandsToWrite ="";
			while(!c.isEmpty()){
				int[] newCom = (int[]) c.remove(0);
				commandsToWrite += "From="+newCom[2]+",To="+newCom[3]+
									",{"+newCom[0]+","+newCom[1]+","+newCom[4]+","+
									newCom[5]+","+newCom[6]+","+newCom[7]+"}\n";
			}
			commandsToWrite += ".";

			try{
				FileWriter fw = new FileWriter(MarvinDialog.pathFile);
				fw.write(commandsToWrite);
				fw.close();
			}
			catch(Exception e){
				System.out.print("Exception trying to close filewriter for path: \n");
				System.out.println(e.toString());
			}

			String outputString;
			outputString = "Ok, roboten vil f beskjed om  g flgende sti: ";
			String outputString2 = "3" + md.getMarvinLocation() ;
			while(!toGoal.isEmpty()){
				if(Integer.parseInt(toGoal.get(toGoal.size()-1).toString()) < 10){
					outputString2 += " -> 30" + toGoal.remove(toGoal.size()-1).toString();
				}
				else{
					outputString2 += " -> 3" + toGoal.remove(toGoal.size()-1).toString();
				}
			}
			outputString2 += " -> " + roomAskedFor;
			while(!fromGoal.isEmpty()){
				if(Integer.parseInt(fromGoal.get(fromGoal.size()-1).toString()) < 10){
					outputString2 += " -> 30" + fromGoal.remove(fromGoal.size()-1).toString();
				}
				else{
					outputString2 += " -> 3" + fromGoal.remove(fromGoal.size()-1).toString();
				}
			}
			outputString2 += " -> 325";

			//return outputString+outputString2;
			return "";
		}
	}

	/**
	*	Method for killing map frames
	*/
	public void killMap(){
		if(mapFrame!=null) mapFrame.dispose();
	}

	/**
	*	Method for checking if room is on this floor
	*/
	private boolean roomOnThisFloor(String s){
		if(s.equals("301") || s.equals("302") || s.equals("303") ||
		   s.equals("304") || s.equals("305") || s.equals("306") ||
		   s.equals("307") || s.equals("308") ||
		   s.equals("310") || s.equals("311") || s.equals("312") ||
		   s.equals("313") || s.equals("314") || s.equals("315") ||
		   s.equals("316") || s.equals("317") || s.equals("320") ||
		   s.equals("321") || s.equals("322") || s.equals("324") ||
		   s.equals("325") || s.equals("326") ||
		   s.equals("330") || s.equals("331") || s.equals("332") ||
		   s.equals("333") || s.equals("334") || s.equals("335") ||
		   s.equals("336") || s.equals("337") || s.equals("342") ||
		   s.equals("343") || s.equals("344") || s.equals("345") ||
		   s.equals("346") || s.equals("347") || s.equals("348") ||
		   s.equals("349") || s.equals("350") ||
		   s.equals("354") || s.equals("356") || s.equals("358") ||
		   s.equals("359") || s.equals("360") || s.equals("361") ||
		   s.equals("363") ) return true;
		else return false;
	}

	/**
	*	Shows map of it-building 3rd floor
	*/
	private void showIt3(String roomNumber){
		JPanel cp = new JPanel();
        if(roomNumber==null) cp.add(new It3Map());
        else cp.add(new It3Map(roomNumber));
        if(mapFrame != null) mapFrame.dispose();
		mapFrame = new JFrame("Map");
		mapFrame.setSize(new Dimension(500,800));
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		mapFrame.setLocation(screenSize.width-500,0);
		mapFrame.setContentPane(cp);
		mapFrame.setFocusableWindowState(false);
		mapFrame.setResizable(false);
		mapFrame.setVisible(true);

		md.setLocation(0,0);
	}

	/**
	*	Shows map of the Glshaugen campus
	*/
	private void showGlos(String building){
		JPanel cp = new JPanel();
		cp.add(new GlosMap(building));
		if(mapFrame != null) mapFrame.dispose();
		mapFrame = new JFrame("Gloshaugen");
		mapFrame.setSize(new Dimension(660,800));
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		mapFrame.setLocation(screenSize.width-660,0);
		mapFrame.setContentPane(cp);
		mapFrame.setFocusableWindowState(false);
		mapFrame.setResizable(false);
		mapFrame.setVisible(true);

		md.setLocation(0,0);
	}

	/**
	*	Shows a map of Trondheim
	*/
	private void showTrondheim(String location){
		JPanel cp = new JPanel();
        cp.add(new TrondheimMap(location));
        if(mapFrame != null) mapFrame.dispose();
		mapFrame = new JFrame("Trondheim");
		mapFrame.setSize(new Dimension(560,400));
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		mapFrame.setLocation(screenSize.width-560,0);
		mapFrame.setContentPane(cp);
		mapFrame.setFocusableWindowState(false);
		mapFrame.setResizable(false);
		mapFrame.setVisible(true);

		md.setLocation(0,0);
	}

	/**
	*	Initializes the hash map of locations
	*/
	private void initMapDescriptions(){
		locationDescriptions = new HashMap();
		locationDescriptions.put(new Integer(25), "Inngangen");
		locationDescriptions.put(new Integer(22), "Rom 322");
		locationDescriptions.put(new Integer(21), "Rom 321");
		locationDescriptions.put(new Integer(20), "Rom 320");
		locationDescriptions.put(new Integer(63), "Rom 363");
		locationDescriptions.put(new Integer(26), "Rom 326");
		locationDescriptions.put(new Integer(24), "Rom 324");
		locationDescriptions.put(new Integer(17), "Rom 317");
		locationDescriptions.put(new Integer(16), "Rom 316");
		locationDescriptions.put(new Integer(15), "Rom 315");
		locationDescriptions.put(new Integer(14), "Rom 314");
		locationDescriptions.put(new Integer(13), "Rom 313");
		locationDescriptions.put(new Integer(12), "Rom 312");
		locationDescriptions.put(new Integer(11), "Rom 311");
		locationDescriptions.put(new Integer(10), "Rom 310");
		locationDescriptions.put(new Integer(8), "Rom 308");
		locationDescriptions.put(new Integer(7), "Rom 307");
		locationDescriptions.put(new Integer(6), "Rom 306");
		locationDescriptions.put(new Integer(5), "Rom 305");
		locationDescriptions.put(new Integer(4), "Rom 304");
		locationDescriptions.put(new Integer(3), "Rom 303");
		locationDescriptions.put(new Integer(2), "Rom 302");
		locationDescriptions.put(new Integer(1), "Rom 301");
		locationDescriptions.put(new Integer(42), "Rom 342");
		locationDescriptions.put(new Integer(431), "Rom 343b");
		locationDescriptions.put(new Integer(43), "Rom 343");
		locationDescriptions.put(new Integer(44), "Rom 344");
		locationDescriptions.put(new Integer(461), "Rom 346b");
		locationDescriptions.put(new Integer(45), "Rom 345");
		locationDescriptions.put(new Integer(46), "Rom 346");
		locationDescriptions.put(new Integer(47), "Rom 347");
		locationDescriptions.put(new Integer(90), "Heisen");
		locationDescriptions.put(new Integer(30), "Rom 330");
		locationDescriptions.put(new Integer(49), "Rom 349");
		locationDescriptions.put(new Integer(48), "Rom 348");
		locationDescriptions.put(new Integer(31), "Rom 331");
		locationDescriptions.put(new Integer(32), "Det lille dametoalettet");
		locationDescriptions.put(new Integer(33), "Rom 333");
		locationDescriptions.put(new Integer(34), "Rom 334");
		locationDescriptions.put(new Integer(35), "Herretoalettet");
		locationDescriptions.put(new Integer(36), "Rom 336");
		locationDescriptions.put(new Integer(37), "Rom 337");
		locationDescriptions.put(new Integer(54), "Rom 354");
		locationDescriptions.put(new Integer(56), "Rom 356");
		locationDescriptions.put(new Integer(58), "Rom 358");
		locationDescriptions.put(new Integer(601), "Rom 360b");
		locationDescriptions.put(new Integer(59), "Rom 359");
		locationDescriptions.put(new Integer(60), "Rom 360");
		locationDescriptions.put(new Integer(61), "Rom 361");
		locationDescriptions.put(new Integer(63), "Rom 363");
		locationDescriptions.put(new Integer(50), "Det store dametoalettet");
	}

	/**
	*	Returns a string that says robot commands have been cleared
	*/
	public String getClearedCommands(){
		return "Alle kommandoer til roboten slettet.";
	}

	/**
	*	Returns a string that says robot location has been changed
	*/
	public String getSetMarvinLocation(int room){
		return ("Marvins lokasjon satt til "+ (room+300));
	}

	/**
	*	Returns a string saying where the robot is
	*/
	public String getLocation(int room){
		return ("Roboten skal vre ved "+(String)locationDescriptions.get(new Integer(room))+". Bruk kommando '|location xxx' hvis dette ikke stemmer.");
	}
}