import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import se.sics.jasper.*;
import java.io.*;
import java.lang.Thread;



/**
 *	This is the main class in Marvin's GUI module. It handles the graphical user interface and communicates
 *	with the other parts of the module, as well as with Telebuster through PrologConnection. It also has some
 *	connection with the Robot's Controller through flat files.
 */
public class MarvinDialog extends JFrame implements ActionListener{

	//Files
	public static final String pathFile = "./robotsim/controllers/path";
	public static final String marvinFile = "./robotsim/controllers/marvin";
	public static final String questionFile = "/directinput1";
	public static final String answerFile = "/directoutput1";
	public static final String telebusterPath = "telebuster.sav";

	//Question types
	public static final String SHOW = "show";
	public static final String PLACE = "place";
	public static final String GO = "go";
	public static final String ACCOMPANY = "accompany";
	public static final String VISIT = "visit";
	public static final String ROUTEPLAN = "routeplan";
	public static final String TELEPHONE = "telephone";
	public static final String ADDRESS = "address";
	public static final String TITLE = "title";
	public static final String DEPARTMENT = "department";
	public static final String YOUAREWELCOME = "youarewelcome";
	public static final String BYE = "bye";
	public static final String DEFAULT = "default";

	//TQL classes
	public static final int WHICH = 0;
	public static final int TEST = 1;
	public static final int NEW = 2;
	public static final int EXPLAIN = 3;
	public static final int DO = 4;
	public static final int DUNNO = 9;
	public static final int NO_QUESTION = 10;

	//Dialog states
	public static final int NEW_CONVERSATION = 0;
	public static final int MORE_INFORMATION = 1;
	public static final int UNKNOWN_LOCATION = 2;

	//Marvin's "memory" of previous conversations
	private Record oldRecord;
	private Location oldLocation;
	private String[] oldTopicAndType;
	private int dialogueState;

	private JTextArea output;
	private JTextField input;
	private JButton answerButton;
	private JScrollPane scrollPane;

	private PrologConnection pc;
	private AnswerGenerator ag;
	private Context context;
	private Tql tql;
	private LanguageUnderstanding lu;

	private String currentAnswer;
	private int screenWidth;

	public boolean details = false;

	/**
	 *	The constructor sets up the GUI. It also makes instances of PrologConnection, AnswerParser and
	 *	AnswerGenerator.
	 */
	public MarvinDialog(){

		//Window
		super("Marvin");
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		screenWidth = screenSize.width;
		if(screenSize.width<=1024)	setSize(new Dimension(550,220));
		else if(screenSize.width==1280) setSize(new Dimension(550,350));
		else if(screenSize.width==1600) setSize(new Dimension(500,420));
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLocation(0,0);
		setResizable(false);

		//Initialize other modules
		context = new Context();
		tql = new Tql();
		ag = new AnswerGenerator(this, context, tql);
		lu = new LanguageUnderstanding(tql, context);

		//Initialize memory
		oldRecord = null;
		oldLocation = null;
		oldTopicAndType = null;
		dialogueState = NEW_CONVERSATION;

		//Output text area
		if(screenSize.width<=1024) output = new JTextArea("", 10, 30);
		else if(screenSize.width==1280) output = new JTextArea("", 18, 30);
		else if(screenSize.width==1600) output = new JTextArea("", 22, 30);
		scrollPane = new JScrollPane(output);
		output.setEditable(false);
		output.setFocusable(false);
		this.getContentPane().add(scrollPane, BorderLayout.NORTH);

		//Input text field
		input = new JTextField();
		input.addActionListener(this);
		input.addMouseListener(new inputMouseListener(this));
		this.getContentPane().add(input, BorderLayout.CENTER);

		setVisible(true);

		//Initialize Prolog Connection
		try{
			Prolog p = Jasper.newProlog(null,null,telebusterPath);
			pc = new PrologConnection(p, this, questionFile, answerFile);
			printMsg("Prolog connection established");
		}
		catch(Exception e){
			printMsg("Error! Could not establish connection with Telebuster!");
		}

		//Initialize Robot location
		setMarvinLocation(25);
		if(details) printMsg(ag.getLocation(getMarvinLocation()));

		//Initialize Simulator
		setRobotNotReady();
		try { Runtime.getRuntime().exec("startrobot.bat"); } catch(IOException ex) { }
		printMsg("Waiting for simulator to get ready");
		while(!robotReady()){
			try{
				Thread.sleep(500);
			}
			catch(Exception e){

			}
		}
		printMsg("Robot simulator ready!\n\n");
		//ag.createTestRun();

		//Make dialog window ready for input
		printMsg("Hei, jeg er Marvin. Hvordan kan jeg hjelpe deg?");
		setFocusable(true);
		requestFocus();
		input.requestFocusInWindow();
		input.setText("Skriv her");
		input.selectAll();
	}

	/**
	*	Gets location from the information file of Marvin
	*
	*	@return			Marvin's location
	*/
	public int getMarvinLocation(){
		int locat = 0;
		try{
			FileReader fr = new FileReader(MarvinDialog.marvinFile);
			File infile = new File(MarvinDialog.marvinFile);
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();

			String marvinFile = "";
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}

			StreamTokenizer st = new StreamTokenizer(new StringReader(marvinFile));
			st.lowerCaseMode(true);
			int tokenType = st.nextToken();
			while(st.ttype != StreamTokenizer.TT_EOF && (st.sval == null || !((st.sval).equals("location")))){
					st.nextToken();
			}
			if(tokenType == StreamTokenizer.TT_EOF) locat = 0;
			else if(tokenType == StreamTokenizer.TT_WORD && st.sval.equals("location")){
				st.nextToken();
				tokenType = st.nextToken();
				if (tokenType == StreamTokenizer.TT_NUMBER) locat = (int)st.nval;
			}
			else locat = 0;
		}
		catch(Exception e){
			System.out.println("Error trying to read Marvin's location from file marvin");
			e.printStackTrace(System.out);
		}

		return locat;
	}

	public void endConversation(){
		pc.processQuery("query('nor 123 adj.').");
	}

	public void selectAllInput(){
		input.selectAll();
	}

	public void storeLocation(Location l){
		oldLocation = l;
	}

	/**
	*	Actions to be performed when MarvinDialog is triggered as an actionListener.
	*
	*	@param e		ActionEvent
	*/
	public void actionPerformed(ActionEvent e){

		//A system call has been entered.
		if(checkSys(input.getText())){
			printMsg(input.getText(), false);
			processSystemCall();
		}

		//New text was entered and Marvin has not offered guidance
		else if(e.getSource()==input){
			printMsg(input.getText(), false);
			processNormalQuestion();
		}
	}

//---------------------------------Private methods--------------------------------------//


	/**
	*	This method is called from actionPerformed, if a normal question has been asked.
	*/
	private void processNormalQuestion(){

		String newText = checkTerm(input.getText());					//Check if there is a termination character

		newText = "query('nor 123 "+ newText + "').";					//Make prolog query

		currentAnswer = pc.processQuery(newText);						//Get answer from Telebuster

		if(details) printMsg(currentAnswer);

		if(currentAnswer.equals("no answer")){							//If no answer from Telebuster

			printMsg("\nTelebuster forstod ikke sprsmlet.");			//Print message
			endConversation();
		}
		else{
			ag.killMap();

			context.parseNewAnswer(currentAnswer);						//Parse context from Telebuster answer

			if(context.personFound()){									//If a new person was found

				oldRecord = context.getRecord();						//Store the new record as old record
			}

			tql.parseNewAnswer(currentAnswer);							//Parse TQL-expression from Telebuster answer

			if(details) printMsg(tql.getElementStrings());

			String[] topicAndType = new String[3];

			if(dialogueState==NEW_CONVERSATION){								//If the dialogue is in 'new conversation'-state

				topicAndType = lu.getTopicAndType();					//Get topic and type from language understanding
				if(details) printMsg("State: New conversation.");
				if(details) printMsg(topicAndType[0]+", "+topicAndType[1]+": "+topicAndType[2]);

			}
			else if(dialogueState==MORE_INFORMATION){							//If the dialogue is in 'get more information'-state

				topicAndType = oldTopicAndType;							//Get topic and type from previous question
				if(details) printMsg("State: More information.");
				if(details) printMsg("Old question:");
				if(details) printMsg(topicAndType[2]);
			}
			else if(dialogueState==UNKNOWN_LOCATION){							//If the dialogue is in 'unknown location'-state

				topicAndType = oldTopicAndType;							//Copy old topic and type

				topicAndType[0] = lu.getTopic();						//Get only the topic from language understanding
				if(details) printMsg("State: Unknown location.");
				if(details) printMsg(topicAndType[2]);
			}
			else{

				printMsg("Programfeil: Dialog-manageren vet ikke hvilken state dialogen er i.");
			}

			////////////////////////// Is topic familiar? ////////////////////////////////////////////////////////////////////
			if(topicAndType[0]==null || topicAndType[0].equals("null") || topicAndType[0].equals("unknown")){						//If topic is unknown

				printMsg("\nDu har spurt om noe jeg ikke skjnner.");
				endConversation();
				dialogueState = NEW_CONVERSATION;
			}

			////////////////////////// Is topic bus? /////////////////////////////////////////////////////////////////////////
			else if(topicAndType[0].equals("bus")){						//If topic is bus

				printMsg("\n"+ag.generateBusAnswer());		//Generate answer, no record needed
				endConversation();
				dialogueState = NEW_CONVERSATION;
			}

			////////////////////////// Is topic person-related? //////////////////////////////////////////////////////////////
			else if(lu.isPersonRelated(topicAndType[0])){					//If topic is person-related

				if(context.personFound()){									//If person is found in new record

					printMsg("\n"+ag.generatePersonAnswer(topicAndType, context.getRecord()));		//Generate answer based on new record
					//oldRecord = context.getRecord();					//Store the new record as old record
					endConversation();									//End conversation in Telebuster
					dialogueState = NEW_CONVERSATION;							//Set dialogue state to 'new conversation'
				}
				else if(context.missingInformation()){						//If Telebuster is missing information about person

					printMsg("\n"+ag.generateFocusAnswer());					//Generate answer based on Telebuster focus
					oldTopicAndType = topicAndType;						//Store the question topic and type
					dialogueState = MORE_INFORMATION;							//Set dialogue state to 'more information'

				}
				else if(oldRecord!=null){								//If there is a previously stored record

					printMsg("\n"+ag.generatePersonAnswer(topicAndType, oldRecord));			//Generate answer based on previously stored record
					endConversation();									//End conversation in Telebuster
					dialogueState = NEW_CONVERSATION;							//Set dialogue state to 'new conversation'
				}
				else if(!(context.getFocus().equals(""))){
					printMsg("\n"+ag.generateFocusAnswer());					//Generate answer based on Telebuster focus
					oldTopicAndType = topicAndType;						//Store the question topic and type
					dialogueState = MORE_INFORMATION;							//Set dialogue state to 'more information'
				}
				else{

					printMsg("\nProgramfeil: Personinformasjon ikke funnet og Telebuster har ikke spurt om mer informasjon");
				}
			}

			////////////////////////// Is topic about unspecific location? ///////////////////////////////////////////////////
			else if(lu.isUnspecificLocation(topicAndType[0])){				//If topic is about an unspecific location

				if(oldLocation!=null){									//If there is a previously stored location

					printMsg("\n"+ag.generateAnswerFromLocation(topicAndType, oldLocation));	//Generate answer based on previously stored location
					endConversation();									//End conversation in Telebuster
					dialogueState = NEW_CONVERSATION;							//Set dialogue state to 'new conversation'
				}
				else{													//If there is no previously stored location

					printMsg("\nHvor mener du?");							//Answer "where?"
					oldTopicAndType = topicAndType;						//Store the question topic and type
					dialogueState = UNKNOWN_LOCATION;							//Set the dialogue state to 'more information'
				}
			}

			///////////////////////// Is topic about something specific? /////////////////////////////////////////////////////
			else if(lu.isSpecific(topicAndType[0])){						//Is topic is about something specific

				printMsg("\n"+ag.generateSpecificAnswer(topicAndType));		//Generate answer, no record needed
				endConversation();										//End conversation in Telebuster
				dialogueState = NEW_CONVERSATION;								//Set the dialogue state to 'new conversation'
			}
		}

		input.selectAll();
	}

	/**
	*	This method is called when a system call has been made.
	*	System calls start with the character '|'
	*
	*	@param s		The system call that has been made.
	*/
	private void processSystemCall(){
		String s = input.getText();
		StreamTokenizer st = new StreamTokenizer(new StringReader(s));
			st.ordinaryChar('|');
		try{
			st.nextToken();
		}
		catch(Exception e){
			System.out.println("Exception trying nextToken() in checkSys()");
			System.out.println(e.toString());
		}
		if(st.ttype == '|'){
			try{
				st.nextToken();
			}
			catch(Exception e){
				System.out.println("Exception trying nextToken() in checkSys()");
				System.out.println(e.toString());
			}

			//User has set a new location for Marvin
			if(st.sval != null && (st.sval).equals("location")){

				while(st.ttype != StreamTokenizer.TT_NUMBER && st.ttype != StreamTokenizer.TT_EOF){
					try{
						st.nextToken();
					}
					catch(Exception e){
						e.printStackTrace(System.out);
					}
					if(st.ttype == StreamTokenizer.TT_EOF){
						printMsg("Not a valid location");
					}
					else if(st.ttype == StreamTokenizer.TT_NUMBER){
						if((int)st.nval > 363 || (int)st.nval < 300) printMsg("Not a valid location");
						else setMarvinLocation(((int)st.nval) - 300);
					}
				}
			}
			//User wants to clear the commands file to the robot
			else if(st.sval != null && (st.sval).equals("clear")){
				try{
					//Clear path file
					FileWriter fw = new FileWriter(MarvinDialog.pathFile);
					fw.write(".");
					fw.close();
					printMsg(ag.getClearedCommands());
				}
				catch(Exception e){
					e.printStackTrace(System.out);
				}
			}
			//Turning details on/off
			else if(st.sval != null && (st.sval).equals("details")){
				if(details){
					details = false;
					printMsg("Details turned off");
				}
				else if(!details){
					details = true;
					printMsg("Details turned on");
				}
			}

			//User has made an illegal system call
			else printMsg("System call not understood");
		}
		else printMsg("System call not understood");

		input.selectAll();
	}

	/**
	*	Checks if the question asked has a terminating character and puts in a
	*	terminating character if not.
	*/
	private String checkTerm(String s){
		StreamTokenizer st = new StreamTokenizer(new StringReader(s));
		st.ordinaryChar('.');
		st.ordinaryChar('?');
		st.ordinaryChar('!');
		try{
			st.nextToken();
		}
		catch(Exception e){
			System.out.println("Exception trying nextToken() in checkTerm()");
			System.out.println(e.toString());
		}
		int previous_type = st.ttype;
		while(!(st.ttype == StreamTokenizer.TT_EOF)){
			previous_type = st.ttype;
			try{
				st.nextToken();
			}
			catch(Exception e){
				System.out.println("Exception trying nextToken() in checkTerm()");
				System.out.println(e.toString());
			}
		}

		if(!(previous_type == '.' || previous_type == '?' || previous_type == '!')){
			s += '.';
		}
		return s;
	}

	/**
	*	Checks if the input text was a system call
	*/
	private boolean checkSys(String s){
		StreamTokenizer st = new StreamTokenizer(new StringReader(s));
		st.ordinaryChar('|');
		try{
			st.nextToken();
		}
		catch(Exception e){
			System.out.println("Exception trying nextToken() in checkSys()");
			System.out.println(e.toString());
		}
		if(st.ttype == '|') return true;
		else return false;
	}

	/**
	*	Simple method to print a string to the textarea.
	*
	*	@param s		String to be printed out.
	*/
	private void printMsg(String s, boolean b){

		String timeString = "";

		//Some code lines to add a nice time stamp
		Calendar time = Calendar.getInstance();
		if(time.get(Calendar.HOUR_OF_DAY) >= 10){
			timeString += time.get(Calendar.HOUR_OF_DAY) + ":";
		}
		else{
			timeString += "0" + time.get(Calendar.HOUR_OF_DAY) + ":";
		}
		if(time.get(Calendar.MINUTE) >= 10){
			timeString += time.get(Calendar.MINUTE) + ":";
		}
		else{
			timeString += "0" + time.get(Calendar.MINUTE) + ":";
		}
		timeString += " | ";

		//Output to the text area
		if(b) output.append(timeString + " Marvin>\n"+ s + "\n");
		else output.append(timeString + " User> "+ s + "\n");

		//Makes the text area scroll down to the bottom
		JScrollBar bar = scrollPane.getVerticalScrollBar();
		bar.setValue(bar.getMaximum());
		output.setCaretPosition(output.getText().length());
	}

	private void printMsg(String s){

		//Output to the text area
		output.append(s + "\n");

		//Makes the text area scroll down to the bottom
		JScrollBar bar = scrollPane.getVerticalScrollBar();
		bar.setValue(bar.getMaximum());
		output.setCaretPosition(output.getText().length());
	}


	private void setRobotNotReady(){
		try{
			String toWrite ="no";

			String marvinFile = "";
			FileReader fr = new FileReader(MarvinDialog.marvinFile);
			File infile = new File(MarvinDialog.marvinFile);
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();
			//Buffer to string
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}
			StringBuffer sb = new StringBuffer(marvinFile);
			int writePosition = 0;

			for(int i=0; i<sb.length(); i++){
				if(i+8 > sb.length()){
					System.out.println("Didn't find 'SimulatorReady' in file marvin");
					i=sb.length();
				}
				else if(sb.substring(i, i+14).equals("SimulatorReady")){
					writePosition = i+15;
					i = sb.length();
				}
			}
			if(writePosition != 0){
				sb = sb.replace(writePosition, writePosition+3, toWrite);
				FileWriter fw = new FileWriter(MarvinDialog.marvinFile);
				fw.write(sb.toString());
				fw.close();
			}
		}
		catch(Exception e){
			System.out.println("Error writing to file marvin");
			e.printStackTrace(System.out);
		}
	}

	/**
	*	Method that sets the Marvin location.
	*
	*	@param location		New location for Marvin.
	*/
	private void setMarvinLocation(int location){
		try{
			String toWrite ="";
			if(location < 10) toWrite = "  "+location;
			else if(location <100) toWrite = " "+location;
			else toWrite = ""+location;

			String marvinFile = "";
			FileReader fr = new FileReader(MarvinDialog.marvinFile);
			File infile = new File(MarvinDialog.marvinFile);
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();
			//Buffer to string
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}
			StringBuffer sb = new StringBuffer(marvinFile);
			int writePosition = 0;

			for(int i=0; i<sb.length(); i++){
				if(i+8 > sb.length()){
					System.out.println("Didn't find 'Location' in file marvin");
					i=sb.length();
				}
				else if(sb.substring(i, i+8).equals("Location")){
					writePosition = i+10;
					i = sb.length();
				}
			}
			if(writePosition != 0){
				sb = sb.replace(writePosition, writePosition+3, toWrite);
				FileWriter fw = new FileWriter(MarvinDialog.marvinFile);
				fw.write(sb.toString());
				fw.close();
				if(details) printMsg(ag.getSetMarvinLocation(location));
			}
		}
		catch(Exception e){
			System.out.println("Error writing to file marvin");
			e.printStackTrace(System.out);
		}
	}

	private boolean robotReady(){
		try{
			FileReader fr = new FileReader(MarvinDialog.marvinFile);
			File infile = new File(MarvinDialog.marvinFile);
			char[] cbuf = new char[(int)infile.length()];
			fr.read(cbuf);
			fr.close();

			String marvinFile = "";
			for(int i=0; i<cbuf.length; i++){
				marvinFile += cbuf[i];
			}

			StreamTokenizer st = new StreamTokenizer(new StringReader(marvinFile));
			while(st.ttype != StreamTokenizer.TT_EOF && !(st.sval != null && st.sval.equals("SimulatorReady"))){
				st.nextToken();
			}
			if(st.ttype == StreamTokenizer.TT_WORD && st.sval.equals("SimulatorReady")){
				st.nextToken();
				st.nextToken();
				if (st.ttype == StreamTokenizer.TT_WORD){
					if(st.sval.equals("yes")) return true;
				}
			}
		}
		catch(Exception e){
			System.out.println("Error trying to read robot ready from file marvin");
			e.printStackTrace(System.out);
		}

		return false;
	}


//---------------------------------Main method------------------------------------------//
	/**
	*	Main method makes an instance of itself.
	*/
	public static void main(String args[]){
		new MarvinDialog();
	}
}

class inputMouseListener implements MouseListener{

	private MarvinDialog md;

	public inputMouseListener(MarvinDialog md){
		super();
		this.md = md;
	}

	public void mouseReleased(MouseEvent e) {
		md.selectAllInput();
	}

	public void mousePressed(MouseEvent e) {

    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {

    }

    public void mouseClicked(MouseEvent e) {

    }

}